# tsn-rt-measures repository

This repository contains various programs I wrote to do TSN measurements

## clockres

clockres is used to determine the resolution of the clock (CLOCK_MONOTONIC), by doing successive
calls to clock_gettime

## packet-exchange

packet-exchange has a client and a server, which exchange UDP ethernet packets using real-time threads,
and can send them on a ETF qdisc by setting a txtime timestamp. Software timestamps can be generated using
the SO_TIMESTAMPING option to measure time spent in kernel.
A Round-Trip Time option also exists to measure RTT of an UDP packet.
XDP sockets can also be used to exchange packets

The client and server programs have a structure similar to cyclictest
They have a real-time thread doing measurements and time sensitive operations 
at regular intervals, and a non real-time thread doing the I/O operations to
report the results shared by the real-time thread.
The real-time thread can be assigned to a CPU and can have it's priority set as
an option, default is to 98.

## Scripts

- run-server, run-client
Front-end interface to the client and server programs from packet-exchange
Uses create-qdisc to setup qdisc
Can use trace-cmd for tracing
Uses /etc/hosts to find IP address

- run-ptp4l, run-phc2sys, run-cyclictest
Easier interfaces to use ptp4l, phc2sys and cyclictest

- test-board-synchro
This script can be used in a setup with two boards connected to one computer
It will synchronize the computer and the boards with linux ptp, and have the computer send timestamps to the boards at a regular interval.
On receiving these timestamps, the boards will toggle one of their GPIO pins at the given timestamp, and the signals can be measured with an oscilloscope, logic analyzer, microcontroller with input capture etc...

- create-qdisc:
Setups ETF or pfifo_fast qdisc, was written for the lime2 board whose
NIC has only one queue, it should not be used for NIC with multiple RX queues

- packet-histogram-stop, packet-histogram-stop
Can automate measures between two boards using ssh and expect scripts
(Needs to be updated: currently has no documentation, and might be broken)

- sudossh
Execute a command with root privileges on the olimex boards without having to enter the password

- exec-ssh-nohup
Execute a command and fork it so that it can still run after ssh is closed

- txtime-stats.py
Used for parsing for the tcpdump option in run-server

- parse-saleae-measures.py
Used to parse measures from the Saleaea logic analyzer


## Measure analysis

The measure-analysis folder contains an automatically generated report with tables and
graphs of all measures done, which can be found in measure-analysis/measure-report.md

The report is generated by a python script on which json formatted sets of
measures can be imported (it can also parse and import cyclictest output)
The imported sets of measures are then stored in the measure folders, and the report
can then be generated with these measures.
Metadatas are included with the measures, such as the kernel version used, the boot parameters passed, various others parameters specific to the measure, etc... Measures measuring the same propriety are grouped together in tables and graphs, and are identified by their diverging metadatas. This is useful to analyse the effect of specific parameters on the  measured propriety.

Report description, metadata description, and measure types description is located into report-info, it is then parsed by the script to generate the report. (The parsing is very sensitive to line breaks)


