#ifndef SEND_PACKET_H
#define SEND_PACKET_H

#include <arpa/inet.h>
#include <errno.h>
#include <error.h>
#include <fcntl.h>
#include <ifaddrs.h>
#include <inttypes.h>
#include <linux/errqueue.h>
#include <linux/ethtool.h>
#include <linux/net_tstamp.h>
#include <linux/sockios.h>
#include <net/if.h>
#include <netdb.h>
#include <netinet/in.h>
#include <poll.h>
#include <pthread.h>
#include <sched.h>
#include <signal.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include "common.h"

typedef struct egress_param {
  int packet_priority;
  size_t tx_buffer_len;
  char server_ip[45];
  char network_if[16];

  int use_etf;
} egress_param_t;

typedef struct egress_stat {

  uint64_t packets_sent;

  uint64_t high_kernel_latency;
  uint64_t invalid_parameter;
  uint64_t missed_deadline;

  int min_interval;
  int avg_interval;
  int max_interval;

} egress_stat_t;

typedef struct egress_info {

  int sock_fd;

  struct sock_txtime sk_txtime;

  char *tx_buffer;

  egress_param_t *params;
  egress_stat_t *stats;

} egress_info_t;

void init_udp_send(egress_info_t *info);
void send_udp_packet(char *data, uint64_t txtime, egress_info_t * info);

#endif
